package sk.telepovsky.android.tutorials.components;

import java.util.ArrayList;

import sk.telepovsky.android.tutorials.R;
import android.app.Activity;
import android.content.Context;
import android.view.Display;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.WindowManager;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.LinearLayout.LayoutParams;
import android.widget.PopupWindow;
import android.widget.TableLayout;
import android.widget.TableRow;
import android.widget.TextView;

public class Menu {

    public enum MenuItemId {
	HELP, ABOUT
    }

    private ArrayList<MenuItem> menuItems;
    private OnMenuItemSelectedListener listener = null;
    private Context context = null;
    private LayoutInflater layoutInflater = null;
    private PopupWindow popupWindow = null;
    private boolean isVisible = false;
    private boolean hideOnSelect = true;
    private int rows = 0;
    private int perLinePortrait = 3;
    private int perLineLandscape = 5;

    /**
     * The interface for returning the item clicked.
     */
    public interface OnMenuItemSelectedListener {
	public void MenuItemSelectedEvent(MenuItem selection);
    }

    /**
     * Constructors
     */
    public Menu(Context context, OnMenuItemSelectedListener listener) {
	menuItems = new ArrayList<MenuItem>();
	this.layoutInflater = ((Activity) context).getLayoutInflater();
	this.context = context;
	this.listener = listener;
    }

    public Menu(Context context, OnMenuItemSelectedListener listener, int perLinePortrait, int perLineLandscape) {
	this(context, listener);
	this.perLinePortrait = perLinePortrait;
	this.perLineLandscape = perLineLandscape;
    }

    /**
     * Use this method to determine if the menu is currently displayed to the
     * user.
     */
    public boolean isShowing() {
	return isVisible;
    }

    /**
     * This setting controls whether or not the menu closes after an item is
     * selected.
     */
    public void setHideOnSelect(boolean hideOnSelect) {
	this.hideOnSelect = hideOnSelect;
    }

    /**
     * Use this method to assign your menu items. You can only call this when
     * the menu is hidden
     */
    public synchronized void setMenuItems(ArrayList<MenuItem> items) throws Exception {
	if (isVisible) {
	    throw new Exception("Menu list may not be modified while menu is displayed.");
	}
	menuItems = items;
    }

    /**
     * Use this method to assign new menu item. You can only call this when the
     * menu is hidden.
     */
    public synchronized void addMenuItem(MenuItem item) {
	if (isVisible)
	    return;

	menuItems.add(item);
    }

    /**
     * Hide menu
     */
    public synchronized void hide() {
	isVisible = false;
	if (popupWindow != null) {
	    popupWindow.dismiss();
	    popupWindow = null;
	}
	return;
    }

    /**
     * Display menu
     */
    public synchronized void show() {

	int itemCount = menuItems.size();

	if (itemCount < 1)
	    return; // no menu items to show

	if (popupWindow != null)
	    return; // already showing

	isVisible = true;

	buildTable(itemCount);
    }

    private void buildTable(int itemCount) {

	Display display = ((WindowManager) context.getSystemService(Context.WINDOW_SERVICE)).getDefaultDisplay();
	boolean isLandscape = display.getWidth() > display.getHeight();

	View mView = layoutInflater.inflate(R.layout.menu, null);
	View root_view = ((Activity) context).findViewById(android.R.id.content);

	popupWindow = new PopupWindow(mView, LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT, false);
	popupWindow.setAnimationStyle(android.R.style.Animation_Toast);
	popupWindow.setWidth(display.getWidth());
	popupWindow.showAtLocation(root_view, Gravity.BOTTOM, 0, 0);

	int divisor = perLinePortrait;

	if (isLandscape)
	    divisor = perLineLandscape;

	int remainder = 0;

	if (itemCount < divisor) {
	    rows = 1;
	    remainder = itemCount;
	} else {
	    rows = (itemCount / divisor);
	    remainder = itemCount % divisor;

	    if (remainder != 0)
		rows++;
	}

	TableLayout table = (TableLayout) mView.findViewById(R.id.custom_menu_table);

	table.removeAllViews();

	for (int i = 0; i < rows; i++) {
	    TableRow row = null;
	    TextView tv = null;
	    ImageView iv = null;

	    // create table row
	    row = new TableRow(context);
	    row.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT));

	    // create menu items
	    for (int j = 0; j < divisor; j++) {
		if (i * divisor + j >= itemCount)
		    break;

		final MenuItem cmi = menuItems.get(i * divisor + j);
		View itemLayout = layoutInflater.inflate(R.layout.menu_item, null);
		
		LinearLayout container = (LinearLayout) itemLayout.findViewById(R.id.ll_item_container);
		if(j == 0) container.setBackgroundResource(android.R.color.transparent);
		
		tv = (TextView) itemLayout.findViewById(R.id.menu_item_caption);
		tv.setText(cmi.getCaption());

		iv = (ImageView) itemLayout.findViewById(R.id.menu_item_icon);
		iv.setImageResource(cmi.getImageResource());

		itemLayout.setOnClickListener(new OnClickListener() {
		    @Override
		    public void onClick(View v) {
			listener.MenuItemSelectedEvent(cmi);

			if (hideOnSelect)
			    hide();
		    }
		});
		row.addView(itemLayout);
	    }
	    table.addView(row);
	}
    }

    /**
     * Toggle menu on user pressing the menu key.
     */
    public void doMenu() {
	if (isShowing())
	    hide();
	else
	    show();
    }

    public void onConfigurationChanged() {
	if (isShowing()) {
	    hide();
	    show();
	}
    }
}